@@ -7,3 +7,8 @@ NULL
 #'
 #' GeoJSON data of Melbourne's Inner suburbs.
 "geo_melbourne"
+
+#' @export
+print.geojson <- function( x, ... ) sapply(x, function(x) cat(x, "\n"))
+
+
