@@ -18,9 +18,11 @@ std::string any_to_string(const T& obj) {
 	return ss.str();
 }
 
-void get_property_types(const Value& v, std::map< std::string, std::string>& property_types);
+void sort_property_names( Rcpp::List& properties, std::unordered_set< std::string >& property_keys);
 
-void get_property_keys(const Value& v, std::set< std::string >& property_keys);
+void get_property_types(const Value& v, std::unordered_map< std::string, std::string>& property_types);
+
+void get_property_keys(const Value& v, std::unordered_set< std::string >& property_keys);
 
 void update_string_vector(Rcpp::List& sf, std::string& key, const std::string& value, const int& row_index);
 
